==> SMILA-1.1-macosx.cocoa.x86_64.zip: org/eclipse/smila/processing/pipelets/SubAttributeExtractorPipelet.java <==
/*******************************************************************************
 * Copyright (c) 2008, 2011 Attensity Europe GmbH and brox IT Solutions GmbH. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse Public License v1.0 which accompanies this
 * distribution, and is available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Daniel Stucky (Attensity Europe GmbH) - initial API and implementation Drazen Cindric (Attensity Europe
 * GmbH) - data model improvements
 *******************************************************************************/

package org.eclipse.smila.processing.pipelets;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.smila.blackboard.Blackboard;
import org.eclipse.smila.blackboard.BlackboardAccessException;
import org.eclipse.smila.datamodel.Any;
import org.eclipse.smila.datamodel.AnyMap;
import org.eclipse.smila.datamodel.AnySeq;
import org.eclipse.smila.datamodel.DataFactory;
import org.eclipse.smila.datamodel.Value;
import org.eclipse.smila.processing.Pipelet;
import org.eclipse.smila.processing.ProcessingException;
import org.eclipse.smila.processing.parameters.ParameterAccessor;
import org.eclipse.smila.processing.util.ProcessingConstants;
import org.eclipse.smila.processing.util.ResultCollector;

/**
 * Extracts Literal values from an attribute that has a nested MObject. The attributes in the nested MObject can have
 * nested MOBjects themselves. To address a attribute in the nested structure a path needs to be specified. The pipelet
 * supports different execution modes: <li>FIRST: selects only the first Literal of the specified attribute</li> <li>
 * LAST: selects only the last Literal of the specified attribute</li> <li>ALL_AS_LIST: selects all Literal values of
 * the specified attribute and returns a list</li> <li>ALL_AS_ONE: selects all Literal values of the specified attribute
 * and concatenates them to a single string, using a seperator (default is blank)</li>
 */
public class SubAttributeExtractorPipelet implements Pipelet {

  /**
   * Name of the property: "inputPath".
   */
  public static final String PROPERTY_INPUT_PATH = "inputPath";

  /**
   * Name of the property: "outputPath".
   */
  public static final String PROPERTY_OUTPUT_PATH = "outputPath";

  /**
   * Name of the property: "mode".
   */
  public static final String PROPERTY_MODE = "mode";

  /**
   * Name of the property: "separator".
   */
  public static final String PROPERTY_SEPARATOR = "separator";

  /**
   * The default separator.
   */
  public static final String DEFAULT_SEPARATOR = " ";

  /**
   * The default path separator.
   */
  public static final String DEFAULT_PATH_SEPARATOR = "/";

  /**
   * local logger.
   */
  private final Log _log = LogFactory.getLog(getClass());

  /**
   * pipelet configuration.
   */
  private AnyMap _configuration;

  /**
   * Enumeration of modes of how to handle multiple values.
   */
  private enum Mode {
    /**
     * Return only the first value.
     */
    FIRST,
    /**
     * Return only the last value.
     */
    LAST,
    /**
     * Return all values as a List.
     */
    ALL_AS_LIST,
    /**
     * Return all values concatenated as one value.
     */
    ALL_AS_ONE;
  }

  /**
   * {@inheritDoc}
   * 
   */
  @Override
  public void configure(final AnyMap configuration) throws ProcessingException {
    _configuration = configuration;
  }

  /**
   * {@inheritDoc}
   * 
   */
  @Override
  public String[] process(final Blackboard blackboard, final String[] recordIds) throws ProcessingException {
    final ParameterAccessor paramAccessor = new ParameterAccessor(blackboard, _configuration);
    final ResultCollector resultCollector =
      new ResultCollector(paramAccessor, _log, ProcessingConstants.DROP_ON_ERROR_DEFAULT);
    if (recordIds != null) {
      for (final String id : recordIds) {
        try {
          paramAccessor.setCurrentRecord(id);
          final String inputPath = paramAccessor.getRequiredParameter(PROPERTY_INPUT_PATH);
          final String[] inputPathNames = inputPath.split(DEFAULT_PATH_SEPARATOR);
          final String outputPath = paramAccessor.getRequiredParameter(PROPERTY_OUTPUT_PATH);
          final Mode mode = Mode.valueOf(paramAccessor.getRequiredParameter(PROPERTY_MODE));
          final String separator = paramAccessor.getParameter(PROPERTY_SEPARATOR, DEFAULT_SEPARATOR);
          final AnyMap metadata = blackboard.getMetadata(id);
          final Any any = getValues(metadata, inputPathNames);
          if (any != null) {
            setValues(metadata, id, any, mode, outputPath, separator);
          }
          resultCollector.addResult(id);
        } catch (final Exception e) {
          resultCollector.addFailedResult(id, e);
        }
      }
    }
    return resultCollector.getResultIds();
  }

  /**
   * get value from input path.
   * 
   * @param metadata
   *          record metadata
   * @param inputPathNames
   *          The path of the input attribute splitted.
   * @return value at input path, or null if path is not valid.
   */
  private Any getValues(final AnyMap metadata, final String[] inputPathNames) {
    AnyMap current = metadata;
    final int lastIndex = inputPathNames.length - 1;
    for (int i = 0; i < lastIndex; i++) {
      Any element = current.get(inputPathNames[i]);
      if (element != null && element.isSeq() && !element.isEmpty()) {
        element = ((AnySeq) element).get(0);
      }
      if (element != null && element.isMap()) {
        current = (AnyMap) element;
      } else {
        return null;
      }
    }
    return current.get(inputPathNames[lastIndex]);
  }

  /**
   * Sets the given values in the output attribute. What values are set depends on the execution mode.
   * 
   * @param metaData
   *          the meta data
   * @param id
   *          the id of the record
   * @param any
   *          the values
   * @param mode
   *          The mode.
   * @param outputPath
   *          The path of the output attribute
   * @param separator
   *          The separator used for mode ALL_AS_ONE.
   * @throws BlackboardAccessException
   *           if any error occurs
   */
  private void setValues(final AnyMap metaData, final String id, final Any any, final Mode mode,
    final String outputPath, final String separator) throws BlackboardAccessException {
    final AnySeq values = metaData.getFactory().createAnySeq();
    for (final Any a : any) {
      if (a.isValue()) {
        values.add(a);
      }
    }
    if (values != null && !values.isEmpty()) {
      switch (mode) {
        case FIRST:
          metaData.put(outputPath, values.get(0));
          break;
        case LAST:
          metaData.put(outputPath, values.get(values.size() - 1));
          break;
        case ALL_AS_LIST:
          metaData.put(outputPath, values);
          break;
        case ALL_AS_ONE:
          metaData.put(outputPath, concat(values, metaData.getFactory(), separator));
          break;
        default:
          _log.error("Error while processing record " + id + ". Invalid mode " + mode);
      }
    }
  }

  /**
   * Concatenates the provided values (in the given {@link AnySeq} object are just {@link Value} instances) as a new
   * String and creates a new {@link Value} instance.
   * 
   * @param values
   *          a List of {@link Value} instances
   * @param factory
   *          the DataFactory used to create the Literal
   * @param separator
   *          The separator used for mode ALL_AS_ONE.
   * @return a {@link Value}
   */
  private Value concat(final AnySeq values, final DataFactory factory, final String separator) {
    final StringBuffer buffer = new StringBuffer();

    for (final Any any : values) {
      final String stringValue = ((Value) any).asString();
      if (buffer.length() > 0) {
        buffer.append(separator);
      }
      buffer.append(stringValue);
    }
    return factory.createStringValue(buffer.toString());
  }

}

==> SMILA-1.3-macosx.cocoa.x86_64.zip: org/eclipse/smila/processing/pipelets/SubAttributeExtractorPipelet.java <==
/*******************************************************************************
 * Copyright (c) 2008, 2013 Empolis Information Management GmbH and brox IT Solutions GmbH. All rights reserved. This
 * program and the accompanying materials are made available under the terms of the Eclipse Public License v1.0 which
 * accompanies this distribution, and is available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * <ul>
 * <li>Daniel Stucky (Attensity Europe GmbH) - initial API and implementation
 * <li>Drazen Cindric (Attensity Europe GmbH) - data model improvements
 * <li>Juergen Schumacher (Empolis Information Management GmbH) - javadoc update
 * </ul>
 *******************************************************************************/

package org.eclipse.smila.processing.pipelets;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.smila.blackboard.Blackboard;
import org.eclipse.smila.blackboard.BlackboardAccessException;
import org.eclipse.smila.datamodel.Any;
import org.eclipse.smila.datamodel.AnyMap;
import org.eclipse.smila.datamodel.AnySeq;
import org.eclipse.smila.datamodel.DataFactory;
import org.eclipse.smila.datamodel.Value;
import org.eclipse.smila.processing.Pipelet;
import org.eclipse.smila.processing.ProcessingException;
import org.eclipse.smila.processing.parameters.ParameterAccessor;
import org.eclipse.smila.processing.util.ProcessingConstants;
import org.eclipse.smila.processing.util.ResultCollector;

/**
 * Extracts simple values (string, number, etc.) from a structured record metadata object, i.e. the object has an
 * attribute that contains a map which can contain further map objects in its attributes. To address an attribute in the
 * nested structure a path needs to be specified, a string of the form "map1/map2/.../valueattribute". If one of the map
 * attributes on the path contains a sequence of maps, only the first of the maps in the sequence is considered. The
 * behavior of the pipelet for a sequence of values in the final value attribute can be configured using the "mode"
 * parameter:
 * <ul>
 * <li>FIRST: selects only the first Literal of the specified attribute</li>
 * <li>LAST: selects only the last Literal of the specified attribute</li>
 * <li>ALL_AS_LIST: selects all Literal values of the specified attribute and returns a list</li>
 * <li>ALL_AS_ONE: selects all Literal values of the specified attribute and concatenates them to a single string, using
 * a separator (default is blank)</li>
 * </ul>
 */
public class SubAttributeExtractorPipelet implements Pipelet {

  /**
   * Name of the property: "inputPath".
   */
  public static final String PROPERTY_INPUT_PATH = "inputPath";

  /**
   * Name of the property: "outputPath".
   */
  public static final String PROPERTY_OUTPUT_PATH = "outputPath";

  /**
   * Name of the property: "mode".
   */
  public static final String PROPERTY_MODE = "mode";

  /**
   * Name of the property: "separator".
   */
  public static final String PROPERTY_SEPARATOR = "separator";

  /**
   * The default separator.
   */
  public static final String DEFAULT_SEPARATOR = " ";

  /**
   * The default path separator.
   */
  public static final String DEFAULT_PATH_SEPARATOR = "/";

  /**
   * local logger.
   */
  private final Log _log = LogFactory.getLog(getClass());

  /**
   * pipelet configuration.
   */
  private AnyMap _configuration;

  /**
   * Enumeration of modes of how to handle multiple values.
   */
  private enum Mode {
    /**
     * Return only the first value.
     */
    FIRST,
    /**
     * Return only the last value.
     */
    LAST,
    /**
     * Return all values as a List.
     */
    ALL_AS_LIST,
    /**
     * Return all values concatenated as one value.
     */
    ALL_AS_ONE;
  }

  /**
   * {@inheritDoc}
   * 
   */
  @Override
  public void configure(final AnyMap configuration) throws ProcessingException {
    _configuration = configuration;
  }

  /**
   * {@inheritDoc}
   * 
   */
  @Override
  public String[] process(final Blackboard blackboard, final String[] recordIds) throws ProcessingException {
    final ParameterAccessor paramAccessor = new ParameterAccessor(blackboard, _configuration);
    final ResultCollector resultCollector =
      new ResultCollector(paramAccessor, _log, ProcessingConstants.DROP_ON_ERROR_DEFAULT);
    if (recordIds != null) {
      for (final String id : recordIds) {
        try {
          paramAccessor.setCurrentRecord(id);
          final String inputPath = paramAccessor.getRequiredParameter(PROPERTY_INPUT_PATH);
          final String[] inputPathNames = inputPath.split(DEFAULT_PATH_SEPARATOR);
          final String outputPath = paramAccessor.getRequiredParameter(PROPERTY_OUTPUT_PATH);
          final Mode mode = Mode.valueOf(paramAccessor.getRequiredParameter(PROPERTY_MODE));
          final String separator = paramAccessor.getParameter(PROPERTY_SEPARATOR, DEFAULT_SEPARATOR);
          final AnyMap metadata = blackboard.getMetadata(id);
          final Any any = getValues(metadata, inputPathNames);
          if (any != null) {
            setValues(metadata, id, any, mode, outputPath, separator);
          }
          resultCollector.addResult(id);
        } catch (final Exception e) {
          resultCollector.addFailedResult(id, e);
        }
      }
    }
    return resultCollector.getResultIds();
  }

  /**
   * get value from input path.
   * 
   * @param metadata
   *          record metadata
   * @param inputPathNames
   *          The path of the input attribute splitted.
   * @return value at input path, or null if path is not valid.
   */
  private Any getValues(final AnyMap metadata, final String[] inputPathNames) {
    AnyMap current = metadata;
    final int lastIndex = inputPathNames.length - 1;
    for (int i = 0; i < lastIndex; i++) {
      Any element = current.get(inputPathNames[i]);
      if (element != null && element.isSeq() && !element.isEmpty()) {
        element = ((AnySeq) element).get(0);
      }
      if (element != null && element.isMap()) {
        current = (AnyMap) element;
      } else {
        return null;
      }
    }
    return current.get(inputPathNames[lastIndex]);
  }

  /**
   * Sets the given values in the output attribute. What values are set depends on the execution mode.
   * 
   * @param metaData
   *          the meta data
   * @param id
   *          the id of the record
   * @param any
   *          the values
   * @param mode
   *          The mode.
   * @param outputPath
   *          The path of the output attribute
   * @param separator
   *          The separator used for mode ALL_AS_ONE.
   * @throws BlackboardAccessException
   *           if any error occurs
   */
  private void setValues(final AnyMap metaData, final String id, final Any any, final Mode mode,
    final String outputPath, final String separator) throws BlackboardAccessException {
    final AnySeq values = metaData.getFactory().createAnySeq();
    for (final Any a : any) {
      if (a.isValue()) {
        values.add(a);
      }
    }
    if (values != null && !values.isEmpty()) {
      switch (mode) {
        case FIRST:
          metaData.put(outputPath, values.get(0));
          break;
        case LAST:
          metaData.put(outputPath, values.get(values.size() - 1));
          break;
        case ALL_AS_LIST:
          metaData.put(outputPath, values);
          break;
        case ALL_AS_ONE:
          metaData.put(outputPath, concat(values, metaData.getFactory(), separator));
          break;
        default:
          _log.error("Error while processing record " + id + ". Invalid mode " + mode);
      }
    }
  }

  /**
   * Concatenates the provided values (in the given {@link AnySeq} object are just {@link Value} instances) as a new
   * String and creates a new {@link Value} instance.
   * 
   * @param values
   *          a List of {@link Value} instances
   * @param factory
   *          the DataFactory used to create the Literal
   * @param separator
   *          The separator used for mode ALL_AS_ONE.
   * @return a {@link Value}
   */
  private Value concat(final AnySeq values, final DataFactory factory, final String separator) {
    final StringBuffer buffer = new StringBuffer();

    for (final Any any : values) {
      final String stringValue = ((Value) any).asString();
      if (buffer.length() > 0) {
        buffer.append(separator);
      }
      buffer.append(stringValue);
    }
    return factory.createStringValue(buffer.toString());
  }

}
